﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Web;

namespace Wisej.Web.Ext.Uploader
{
    public class ChunkUploadHandler : HttpTaskAsyncHandler
    {
        private static Dictionary<string, FileState> locks = new Dictionary<string, FileState>();
        private static object myLock = new object();
        public override bool IsReusable
        {
            get { return true; }
        }

        public override async Task ProcessRequestAsync(HttpContext context)
        {
            var request = context.Request;

            int chunkNumber = int.Parse(request.Params["resumableChunkNumber"].Split(',')[0]);
            int totalChunks = int.Parse(request.Params["resumableTotalChunks"].Split(',')[0]);
            int chunkSize = int.Parse(request.Params["resumableChunkSize"].Split(',')[0]);
            long totalSize = long.Parse(request.Params["resumableTotalSize"].Split(',')[0]);
            var uniqueId = request.Params["resumableIdentifier"].Split(',')[0];
            var fileName = request.Params["resumableFilename"].Split(',')[0];
            var relativePath = request.Params["resumableRelativePath"].Split(',')[0];

            var tempFolder = Path.Combine(request.PhysicalApplicationPath, "Temp");
            var filePath = Path.Combine(tempFolder, uniqueId);

            if (request.RequestType == "GET")
            {
                context.Response.StatusCode = (int)HttpStatusCode.PartialContent;
                context.Response.Flush();
                return;
            }         

            try
            {
                byte[] buffer = new byte[request.ContentLength];
                await request.InputStream.ReadAsync(buffer, 0, request.ContentLength);

                var offSet = (chunkNumber - 1) * chunkSize;

                var fileState = GetLock(filePath);

                var locker = fileState.locker;

                await locker.LockAsync(async () =>
                {
                    using (var fs = new FileStream(filePath, FileMode.OpenOrCreate, FileAccess.Write, FileShare.ReadWrite | FileShare.Delete))
                    {
                        if (fs.Length < offSet)
                        {
                            fs.SetLength(offSet);
                        }
                        fs.Position = offSet;

                        await fs.WriteAsync(buffer, 0, request.ContentLength);
                        fileState.DoneList.Add(chunkNumber);
                    }
                }, TimeSpan.FromSeconds(30));

                if (fileState.DoneList.Count == totalChunks)
                {
                    locks.Remove(filePath);
                }

                context.Response.Write("OK");
                context.Response.StatusCode = (int)HttpStatusCode.OK;
                context.Response.Flush();
            }
            catch (Exception ex)
            {
                context.Response.Write(ex.ToString());
                context.Response.StatusCode = (int)HttpStatusCode.InternalServerError;
                context.Response.Flush();
            }
        }

        private FileState GetLock(string name)
        {
            lock (myLock)
            {
                if (locks.ContainsKey(name))
                {
                    return locks[name];
                }

                var readerWriterLock = new FileState();
                locks.Add(name, readerWriterLock);
                return readerWriterLock;
            }
        }
    }

    internal class FileState
    {
        public AsyncLocker locker = new AsyncLocker();
        public ConcurrentBag<int> DoneList = new ConcurrentBag<int>();
    }

    internal class AsyncLocker
    {
        private readonly SemaphoreSlim _semaphore = new SemaphoreSlim(1, 1);

        public async Task LockAsync(Func<Task> worker, TimeSpan timeSpan)
        {
            await _semaphore.WaitAsync(timeSpan);
            try
            {
                await worker();
            }
            finally
            {
                _semaphore.Release();
            }
        }
    }
}
