﻿using Progression.Extras;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using Wisej.Web;

namespace Wisej.Web.Ext.Uploader
{
    public partial class UploadControl : Wisej.Web.UserControl
    {
        public event CancelEventHandler FileOk;
        public event EventHandler<UploaderErrorEventArgs> Error;
        private ETACalculator etaCalculator = null;
        public List<Uploader.UploadFileInfo> Files
        {
            get
            {
                return this.uploader1.Files;
            }
        }
        public Uploader.UploadFileInfo File
        {
            get
            {
                return Files.SingleOrDefault();
            }
        }
        public UploadControl()
        {
            InitializeComponent();

            this.lbFileName.Text = string.Empty;
            this.lbStatus.Text = string.Empty;

            this.Disposed += UploadControl_Disposed;
            etaCalculator = new ETACalculator(5, 5);
        }

        private void UploadControl_Disposed(object sender, EventArgs e)
        {
            uploader1?.Cancel();
            uploader1?.Dispose();
            uploader1 = null;
        }

        private void uploader1_UploadProgress(object sender, UploaderProgressEventArgs e)
        {
            var percentComplete = (double)e.Loaded / (double)e.Total;
            this.etaCalculator.Update((float)percentComplete / 100);
            this.progressBar1.Value = (int)percentComplete;

            var transferRate = e.TransferRate;
            if (percentComplete >= 100)
            {
                transferRate = 0;
            }

            this.lbStatus.Text = $"{Environment.NewLine}Status{Environment.NewLine}" + (etaCalculator.ETAIsAvailable ? string.Format("{0:00}% done\r\n{1:0.0} {2} remaining\r\n {3} transfer rate", (float)percentComplete, etaCalculator.ETR.TotalSeconds > 60 ? etaCalculator.ETR.TotalMinutes : etaCalculator.ETR.TotalSeconds, etaCalculator.ETR.TotalSeconds > 60 ? "minutes" : "seconds", FormatSize(transferRate)) : "Calculating...");
        }

        public void Cancel()
        {
            uploader1?.Cancel();
            uploader1?.Dispose();
            uploader1 = null;
        }

        private void uploader1_OnCancelled(object sender, EventArgs e)
        {
            this.btnUpload.Enabled = true;
            this.uploader1.Enabled = true;
        }

        private void uploader1_OnFileReady(object sender, EventArgs e)
        {
            this.btnUpload.Enabled = true;
            this.progressBar1.Value = 0;
            this.lbStatus.Text = string.Empty;
            this.lbFileName.Text = $"File Name:{this.uploader1.FileName}{Environment.NewLine}Size:{FormatSize(this.uploader1.FileSize)}";
        }

        private void uploader1_OnStarted(object sender, EventArgs e)
        {
            this.btnCancel.Enabled = true;
            this.btnUpload.Enabled = false;
            this.etaCalculator.Reset();
        }

        string FormatSize(long val)
        {
            string size = "0";

            if (val < 1)
            {
                return size;
            }

            if ((val / 1024 / 1024) > 1024)
                size = (val / 1024 / 1024 / 1024) + " gb";
            if ((val / 1024) > 1024)
                size = (val / 1024 / 1024) + " mb";
            else if (val > 1024)
                size = (val / 1024) + " kb";
            else
                size = val + " b";
            return size;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.btnCancel.Enabled = false;
            this.uploader1.Cancel();
        }

        private void btnUpload_Click(object sender, EventArgs e)
        {
            this.btnUpload.Enabled = false;
            this.uploader1.Enabled = false;
            this.uploader1.StartUpload();
        }

        private void uploader1_FileOk(object sender, CancelEventArgs e)
        {
            this.btnUpload.Enabled = true;
            this.uploader1.Enabled = true;
            this.btnCancel.Enabled = false;
            this.FileOk?.Invoke(sender, e);
        }

        private void uploader1_Error(object sender, UploaderErrorEventArgs e)
        {
            this.btnUpload.Enabled = true;
            this.uploader1.Enabled = true;
            this.btnCancel.Enabled = false;
            this.lbStatus.Text = e.Message;
            this.Error?.Invoke(sender, e);
            Application.Update(this);
        }
    }
}
